<?php

namespace Drupal\cmlmigrations\Hook;

use Drupal\Component\Serialization\Json;
use Drupal\paragraphs\Entity\Paragraph;

/**
 * @file
 * Contains \Drupal\cmlmigrations\Hook\CommerceProductVariationPresave.
 */

/**
 * Hook presave.
 */
class CommerceProductVariationPresave {

  /**
   * Hook.
   */
  public static function hook($variation) {
    if ($variation->hasField('field_json_stores')) {
      $stores = JSON::decode($variation->field_json_stores->value);
      if (!empty($stores)) {
        $mapStores = self::queryMap('migrate_map_cml_taxonomy_stores');
        $paragraphs = [];
        foreach ($stores as $key => $value) {
          if (!empty($mapStores[$key])) {
            $paragraphs[] = self::hasStore($mapStores[$key], $value, $variation->id());
          }
        }
        $variation->set('field_stocks', $paragraphs);
      }
    }
  }

  /**
   * HasStore.
   */
  private static function hasStore($store_id, $quantity, $variationId) {
    $result = FALSE;
    $storage = \Drupal::entityTypeManager()->getStorage('paragraph');
    $ids = $storage
      ->getQuery()
      ->condition('status', 1)
      ->condition('type', 'stores')
      ->condition('parent_type', 'commerce_product_variation')
      ->condition('parent_field_name', 'field_stores')
      ->condition('parent_id', $variationId)
      ->condition('field_stores_store', $store_id)
      ->execute();
    if ($ids) {
      foreach ($storage->loadMultiple($ids) as $paragraph) {
        $paragraph->set('field_stores_stock', $quantity);
        $paragraph->save();
      }
    }
    else {
      $paragraph = Paragraph::create([
        'type' => 'stores',
        'parent_id' => $variationId,
        'parent_type' => 'commerce_product_variation',
        'parent_field_name' => 'field_stores',
        'field_stores_stock' => $quantity,
        'field_stores_store' => [
          'target_id' => $store_id,
        ],
      ]);
      $paragraph->save();
    }
    $result = [
      'target_id' => $paragraph->id(),
      'target_revision_id' => $paragraph->getRevisionId(),
    ];
    return $result;
  }

  /**
   * Query MAP.
   */
  private static function queryMap($table, $subs = FALSE) {
    $data = [];
    $db = \Drupal::database();
    if (!$db->schema()->tableExists($table)) {
      return [];
    }
    $query = $db->select($table, 'map')->fields('map', [
      'sourceid1',
      'destid1',
    ]);
    $res = $query->execute();
    if ($res) {
      foreach ($res as $key => $row) {
        $data[$row->sourceid1] = $row->destid1;
      }
    }
    return $data;
  }

}
